#
# $Id: UNIVERSAL.pl 32 2005-11-09 10:32:15Z manu $
#

# Copyright I.N.R.A. - C.N.R.S. 

# emmanuel.courcelle@toulouse.inra.fr
# Inspired by "Object Oriented Perl", by Damian Conway

# This software is a perl module whose purpose is to help you writing 
# your own scripts

# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# http://www.cecill.info" 

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=head1 NAME

UNIVERSAL - Functions added to UNIVERSAL: every object derives from it

=head1 DESCRIPTION
 
 At the beginning of your code, write:

 require "UNIVERSAL.pl";

 This file defines 2 methods for the UNIVERSAL module: New and Clone.
 If object B derives from object A, inheritance "tree" looks like:
 
   UNIVERSAL     
       ^
       |
       |
    Object_A
       ^
       |
       |
    Object_B
    
 Then, Object_A and Object_B DO NOT define any constructor, they rather use the UNIVERSAL New method
 However, Object_A and Object_B could define a procedure named _Init, written as follows:
 
 sub _Init
 {
     my $self = shift;
     my @args = @_;
     my @super_args = ...;
     $self->SUPER::_Init(@super_args);
     ...    # More initialization
 }

 The method Clone lay be used to create a new object, initialized from the current object.
 WARNING !!! PLEASE DO NOT USE THIS METHOD if some attribute is a ref towards some other variable or object.
 
=cut

use strict;
use warnings;

=head2 UNIVERSAL::New

 Title    : New
 Usage    : The constructor
 Access   : Public
 Function : The internal hash ($self) is created and blessed.
            The protected function _Init is then called, letting the derived classes
            to perform their own initialization
 Args     : the class, followed by object-specific arguments
 Returns  : The object
 Globals  : none

=cut

sub UNIVERSAL::New
{
    my ($class, @args) = @_;
    my $self = bless {}, $class;
    $self->_Init(@args);
    return $self;
}

=head2 UNIVERSAL::Clone

 Title    : Clone
 Usage    : Clone (=copy) the object
 Access   : Public
 Function : A new hash is created and initialized from $self, and blessed
 Args     : none (-- or just the object)
 Returns  : The new object
 Globals  : none

=cut

sub UNIVERSAL::Clone
{
    my $self    = shift;
    my $class   = ref($self);
    my $new_obj = bless {%{$self}}, $class;
    return $new_obj;
}

=head2 UNIVERSAL::_Init

 Title    : _Init
 Usage    : Just to make the compiler happy in case the derived classes did not define _Init
 Access   : Protected
 Procedure: Does not do anything
 Args     : none
 Returns  : nothing
 Globals  : none

=cut

sub UNIVERSAL::_Init { }

1;

